require 'installer/action'
require 'installer/errors'

class Installer::Planner::Step::VerifyPuppet < Installer::Planner::Step
  def initialize(infra, data={}, pre=nil, post=nil)
    super

    @host = @infra.hosts[data[:host]] if data[:host]
  end

  def perform_step(&execute)
    puppettest = Installer::Action::Execute.new(:host => @host, :sudo => true, :cmd => "/opt/puppet/bin/puppet agent --test")
    yield puppettest

    if puppettest.success? || puppettest.results[@host.hostname][:exit_status] == 2
      info(@host.hostname, "The puppet agent ran successfully.")
    else
      error(@host.hostname, "The puppet agent test run failed.")

      # Verify all services for a host are running locally
      @host.installed_services.each do |service_name|
        puppetservice = Installer::Action::Execute.new(:host => @host, :sudo => true, :cmd => "/opt/puppet/bin/puppet resource service #{service_name}")
        yield puppetservice

        if puppetservice.success?
          stdout = puppetservice.results[@host.hostname][:stdout]
          service_match = /service { '(.*)'.*ensure => '(.*)'.*}/m.match(stdout)
          if service_match && service_match[2] == 'running'
            info(@host.hostname, "#{service_match[1]} service is running.")
          else
            error(@host.hostname, "#{service_match[1]} service is NOT running.")
          end
        else
          error(@host.hostname, "Failed to check the status of #{service_match[1]} service.")
        end
      end

      @host.outbound.each do |h|
        rolehosts = @infra.find_hosts_with_roles([h[:role]])
        if rolehosts.any?
          rolehost = rolehosts.first.hostname
          tcpcheck = Installer::Action::Execute.new(:host => @host, :cmd => "echo 1 > /dev/tcp/#{rolehost}/#{h[:port]}")
          yield tcpcheck

          if tcpcheck.success?
            info(@host.hostname, "Successfully connected to #{rolehost}:#{h[:port]}.")
          else
            error(@host.hostname, "Could not connect to #{rolehost}:#{h[:port]}.")
          end
        else
          error(@host.hostname, "Could not find a host with role #{h[:role]}.")
        end
      end

      @host.inbound.each do |h|
        rolehosts = @infra.find_hosts_with_roles([h[:role]])
        if rolehosts.any?
          rolehost = rolehosts.first
          tcpcheck = Installer::Action::Execute.new(:host => rolehost, :cmd => "echo 1 > /dev/tcp/#{@host.hostname}/#{h[:port]}")
          yield tcpcheck

          if tcpcheck.success?
            info(@host.hostname, "Is allowing connections from #{rolehost} on port #{h[:port]}.")
          else
            error(@host.hostname, "Is refusing connections from #{rolehost} on port #{h[:port]}.")
          end
        else
          error(@host.hostname, "Could not find a host with role #{h[:role]}.")
        end
      end

      raise Installer::RemoteError, "Puppet Test Run Failed."
    end
  end

  def name
    "verify_puppet_#{@host}"
  end

  def description
    "Verifying that Puppet Enterprise is functioning on #{@host}."
  end
end
